@@ -207,7 +207,6 @@ public:
 
         uint options = 0x0;
 
-        options |= PLUGIN_OPTION_MAP_PROGRAM_CHANGES;
         options |= PLUGIN_OPTION_USE_CHUNKS;
 
         if (fInstance->getNumPrograms() > 1)
@@ -1101,20 +1100,11 @@ public:
 
                     if (status == MIDI_STATUS_NOTE_ON)
                     {
-                        pData->postponeRtEvent(kPluginPostRtEventNoteOn,
-                                               true,
-                                               event.channel,
-                                               midiData[1],
-                                               midiData[2],
-                                               0.0f);
+                        pData->postponeNoteOnRtEvent(true, event.channel, midiData[1], midiData[2]);
                     }
                     else if (status == MIDI_STATUS_NOTE_OFF)
                     {
-                        pData->postponeRtEvent(kPluginPostRtEventNoteOff,
-                                               true,
-                                               event.channel,
-                                               midiData[1],
-                                               0, 0.0f);
+                        pData->postponeNoteOffRtEvent(true, event.channel, midiData[1]);
                     }
                 } break;
                 } // switch (event.type)
@@ -1387,7 +1377,28 @@ public:
                 fDesc.name = label;
         }
 
-        fFormatManager.addDefaultFormats();
+        /**/ if (std::strcmp(format, "AU") == 0)
+        {
+#if JUCE_PLUGINHOST_AU
+            fFormatManager.addFormat(new juce::AudioUnitPluginFormat());
+#endif
+        }
+        else if (std::strcmp(format, "VST2") == 0)
+        {
+#if JUCE_PLUGINHOST_VST
+            fFormatManager.addFormat(new juce::VSTPluginFormat());
+#endif
+        }
+        else if (std::strcmp(format, "VST3") == 0)
+        {
+#if JUCE_PLUGINHOST_VST3
+            fFormatManager.addFormat(new juce::VST3PluginFormat());
+#endif
+        }
+        else
+        {
+            fFormatManager.addDefaultFormats();
+        }
 
         {
             juce::OwnedArray<juce::PluginDescription> pluginDescriptions;
@@ -1398,12 +1409,18 @@ public:
 
                 for (int i = 0; i < fFormatManager.getNumFormats(); ++i)
                 {
+                    juce::AudioPluginFormat* const apformat = fFormatManager.getFormat(i);
+                    CARLA_SAFE_ASSERT_CONTINUE(apformat != nullptr);
+
+                    carla_debug("Trying to load '%s' plugin with format '%s'", fileOrIdentifier.toRawUTF8(), apformat->getName().toRawUTF8());
+
                     try {
-                        plist.scanAndAddFile(fileOrIdentifier, true, pluginDescriptions, *fFormatManager.getFormat(i));
+                        plist.scanAndAddFile(fileOrIdentifier, true, pluginDescriptions, *apformat);
                     } CARLA_SAFE_EXCEPTION_CONTINUE("scanAndAddFile")
 
                     if (sac.wasTriggered())
                     {
+                        carla_stderr("WARNING: Caught exception while scanning file, will not load this plugin");
                         pluginDescriptions.clearQuick(false);
                         break;
                     }
@@ -1435,7 +1452,10 @@ public:
             } CARLA_SAFE_EXCEPTION("createPluginInstance")
 
             if (sac.wasTriggered())
+            {
                 fInstance = nullptr;
+                carla_stderr("WARNING: Caught exception while instantiating, will not load this plugin");
+            }
         }
 
         if (fInstance == nullptr)
@@ -1497,11 +1517,9 @@ public:
                 pData->options |= PLUGIN_OPTION_SKIP_SENDING_NOTES;
         }
 
-        if (fInstance->getNumPrograms() > 1 && ((pData->options & PLUGIN_OPTION_SEND_PROGRAM_CHANGES) == 0))
-        {
+        if (fInstance->getNumPrograms() > 1 && (pData->options & PLUGIN_OPTION_SEND_PROGRAM_CHANGES) == 0)
             if (isPluginOptionEnabled(options, PLUGIN_OPTION_MAP_PROGRAM_CHANGES))
                 pData->options |= PLUGIN_OPTION_MAP_PROGRAM_CHANGES;
-        }
 
         return true;
     }
