@@ -25,11 +25,16 @@
 
 #include "water/files/File.h"
 
+#ifdef CARLA_OS_MAC
+# include "CarlaMacUtils.hpp"
+#endif
+
 #include <string>
 #include <vector>
 
 #define URI_CARLA_ATOM_WORKER_IN   "http://kxstudio.sf.net/ns/carla/atomWorkerIn"
 #define URI_CARLA_ATOM_WORKER_RESP "http://kxstudio.sf.net/ns/carla/atomWorkerResp"
+#define URI_CARLA_PARAMETER_CHANGE "http://kxstudio.sf.net/ns/carla/parameterChange"
 
 using water::File;
 
@@ -77,7 +82,8 @@ enum CarlaLv2URIDs {
     kUridLogTrace,
     kUridLogWarning,
     kUridPatchSet,
-    kUridPatchPoperty,
+    kUridPatchProperty,
+    kUridPatchSubject,
     kUridPatchValue,
     // time base type
     kUridTimePosition,
@@ -102,6 +108,7 @@ enum CarlaLv2URIDs {
     // custom carla props
     kUridCarlaAtomWorkerIn,
     kUridCarlaAtomWorkerResp,
+    kUridCarlaParameterChange,
     kUridCarlaTransientWindowId,
     // count
     kUridCount
@@ -219,6 +226,45 @@ struct Lv2PluginOptions {
     }
 };
 
+// -------------------------------------------------------------------------------------------------------------------
+
+static void initAtomForge(LV2_Atom_Forge& atomForge) noexcept
+{
+    carla_zeroStruct(atomForge);
+
+    atomForge.Bool     = kUridAtomBool;
+    atomForge.Chunk    = kUridAtomChunk;
+    atomForge.Double   = kUridAtomDouble;
+    atomForge.Float    = kUridAtomFloat;
+    atomForge.Int      = kUridAtomInt;
+    atomForge.Literal  = kUridAtomLiteral;
+    atomForge.Long     = kUridAtomLong;
+    atomForge.Object   = kUridAtomObject;
+    atomForge.Path     = kUridAtomPath;
+    atomForge.Property = kUridAtomProperty;
+    atomForge.Sequence = kUridAtomSequence;
+    atomForge.String   = kUridAtomString;
+    atomForge.Tuple    = kUridAtomTuple;
+    atomForge.URI      = kUridAtomURI;
+    atomForge.URID     = kUridAtomURID;
+    atomForge.Vector   = kUridAtomVector;
+
+#if defined(__clang__)
+# pragma clang diagnostic push
+# pragma clang diagnostic ignored "-Wdeprecated-declarations"
+#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wdeprecated-declarations"
+#endif
+    atomForge.Blank    = kUridAtomBlank;
+    atomForge.Resource = kUridAtomResource;
+#if defined(__clang__)
+# pragma clang diagnostic pop
+#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
+# pragma GCC diagnostic pop
+#endif
+}
+
 // --------------------------------------------------------------------------------------------------------------------
 
 class CarlaLv2Client : public CarlaBridgeFormat
@@ -473,6 +519,11 @@ public:
         // ------------------------------------------------------------------------------------------------------------
         // open DLL
 
+#ifdef CARLA_OS_MAC
+        // Binary might be in quarentine due to Apple stupid notarization rules, let's remove that if possible
+        CarlaBackend::removeFileFromQuarantine(fRdfUiDescriptor->Binary);
+#endif
+
         if (! libOpen(fRdfUiDescriptor->Binary))
         {
             carla_stderr("Failed to load UI binary, error was:\n%s", libError());
@@ -587,6 +638,80 @@ public:
         fDescriptor->port_event(fHandle, index, sizeof(float), kUridNull, &value);
     }
 
+    void dspParameterChanged(const char* const uri, const float value) override
+    {
+        CARLA_SAFE_ASSERT_RETURN(fHandle != nullptr,)
+        CARLA_SAFE_ASSERT_RETURN(fDescriptor != nullptr,);
+
+        if (fDescriptor->port_event == nullptr)
+            return;
+
+        uint32_t parameterId = UINT32_MAX;
+
+        for (uint32_t i=0; i < fRdfDescriptor->ParameterCount; ++i)
+        {
+            const LV2_RDF_Parameter& rdfParam(fRdfDescriptor->Parameters[i]);
+
+            if (std::strcmp(rdfParam.URI, uri) == 0)
+            {
+                parameterId = i;
+                break;
+            }
+        }
+
+        if (parameterId == UINT32_MAX)
+            return;
+
+        uint8_t atomBuf[256];
+        LV2_Atom_Forge atomForge;
+        initAtomForge(atomForge);
+        lv2_atom_forge_set_buffer(&atomForge, atomBuf, sizeof(atomBuf));
+
+        LV2_Atom_Forge_Frame forgeFrame;
+        lv2_atom_forge_object(&atomForge, &forgeFrame, kUridNull, kUridPatchSet);
+
+        lv2_atom_forge_key(&atomForge, kUridCarlaParameterChange);
+        lv2_atom_forge_bool(&atomForge, true);
+
+        lv2_atom_forge_key(&atomForge, kUridPatchProperty);
+        lv2_atom_forge_urid(&atomForge, getCustomURID(uri));
+
+        lv2_atom_forge_key(&atomForge, kUridPatchValue);
+
+        switch (fRdfDescriptor->Parameters[parameterId].Type)
+        {
+        case LV2_PARAMETER_TYPE_BOOL:
+            lv2_atom_forge_bool(&atomForge, value > 0.5f);
+            break;
+        case LV2_PARAMETER_TYPE_INT:
+            lv2_atom_forge_int(&atomForge, static_cast<int32_t>(value + 0.5f));
+            break;
+        case LV2_PARAMETER_TYPE_LONG:
+            lv2_atom_forge_long(&atomForge, static_cast<int64_t>(value + 0.5f));
+            break;
+        case LV2_PARAMETER_TYPE_FLOAT:
+            lv2_atom_forge_float(&atomForge, value);
+            break;
+        case LV2_PARAMETER_TYPE_DOUBLE:
+            lv2_atom_forge_double(&atomForge, value);
+            break;
+        default:
+            carla_stderr2("dspParameterChanged called for invalid parameter, abort!");
+            return;
+        }
+
+        lv2_atom_forge_pop(&atomForge, &forgeFrame);
+
+        LV2_Atom* const atom((LV2_Atom*)atomBuf);
+        CARLA_SAFE_ASSERT(atom->size < sizeof(atomBuf));
+
+        fDescriptor->port_event(fHandle,
+                                fControlDesignatedPort,
+                                lv2_atom_total_size(atom),
+                                kUridAtomTransferEvent,
+                                atom);
+    }
+
     void dspProgramChanged(const uint32_t index) override
     {
         carla_stderr2("dspProgramChanged(%i) - not handled", index);
@@ -824,7 +949,7 @@ public:
 
         for (uint32_t i=0; i < fRdfDescriptor->ParameterCount; ++i)
         {
-            if (fRdfDescriptor->Parameters[i].Type != LV2_PARAMETER_PATH)
+            if (fRdfDescriptor->Parameters[i].Type != LV2_PARAMETER_TYPE_PATH)
                 continue;
             if (std::strcmp(fRdfDescriptor->Parameters[i].URI, uri) != 0)
                 continue;
@@ -1144,7 +1269,9 @@ private:
         if (std::strcmp(uri, LV2_PATCH__Set) == 0)
             return kUridPatchSet;
         if (std::strcmp(uri, LV2_PATCH__property) == 0)
-            return kUridPatchPoperty;
+            return kUridPatchProperty;
+        if (std::strcmp(uri, LV2_PATCH__subject) == 0)
+            return kUridPatchSubject;
         if (std::strcmp(uri, LV2_PATCH__value) == 0)
             return kUridPatchValue;
 
@@ -1191,6 +1318,8 @@ private:
             return kUridCarlaAtomWorkerIn;
         if (std::strcmp(uri, URI_CARLA_ATOM_WORKER_RESP) == 0)
             return kUridCarlaAtomWorkerResp;
+        if (std::strcmp(uri, URI_CARLA_PARAMETER_CHANGE) == 0)
+            return kUridCarlaParameterChange;
         if (std::strcmp(uri, LV2_KXSTUDIO_PROPERTIES__TransientWindowId) == 0)
             return kUridCarlaTransientWindowId;
 
@@ -1277,8 +1406,10 @@ private:
         // Patch types
         case kUridPatchSet:
             return LV2_PATCH__Set;
-        case kUridPatchPoperty:
+        case kUridPatchProperty:
             return LV2_PATCH__property;
+        case kUridPatchSubject:
+            return LV2_PATCH__subject;
         case kUridPatchValue:
             return LV2_PATCH__value;
 
@@ -1325,6 +1456,8 @@ private:
             return URI_CARLA_ATOM_WORKER_IN;
         case kUridCarlaAtomWorkerResp:
             return URI_CARLA_ATOM_WORKER_RESP;
+        case kUridCarlaParameterChange:
+            return URI_CARLA_PARAMETER_CHANGE;
         case kUridCarlaTransientWindowId:
             return LV2_KXSTUDIO_PROPERTIES__TransientWindowId;
         }
