@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 
 # Common Carla code
-# Copyright (C) 2011-2020 Filipe Coelho <falktx@falktx.com>
+# Copyright (C) 2011-2021 Filipe Coelho <falktx@falktx.com>
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License as
@@ -71,7 +71,7 @@ if WINDOWS:
 # ------------------------------------------------------------------------------------------------------------
 # Set Version
 
-VERSION = "2.2.0"
+VERSION = "2.3.0"
 
 # ------------------------------------------------------------------------------------------------------------
 # Set TMP
@@ -184,6 +184,7 @@ CARLA_KEY_MAIN_PRO_THEME_COLOR  = "Main/ProThemeColor"   # str
 CARLA_KEY_MAIN_REFRESH_INTERVAL = "Main/RefreshInterval" # int
 CARLA_KEY_MAIN_CONFIRM_EXIT     = "Main/ConfirmExit"     # bool
 CARLA_KEY_MAIN_SHOW_LOGS        = "Main/ShowLogs"        # bool
+CARLA_KEY_MAIN_SYSTEM_ICONS     = "Main/SystemIcons"     # bool
 CARLA_KEY_MAIN_EXPERIMENTAL     = "Main/Experimental"    # bool
 
 CARLA_KEY_CANVAS_THEME             = "Canvas/Theme"           # str
@@ -259,6 +260,7 @@ CARLA_DEFAULT_MAIN_PRO_THEME_COLOR  = "Black"
 CARLA_DEFAULT_MAIN_REFRESH_INTERVAL = 20
 CARLA_DEFAULT_MAIN_CONFIRM_EXIT     = True
 CARLA_DEFAULT_MAIN_SHOW_LOGS        = bool(not WINDOWS)
+CARLA_DEFAULT_MAIN_SYSTEM_ICONS     = False
 CARLA_DEFAULT_MAIN_EXPERIMENTAL     = False
 
 # Canvas
@@ -566,7 +568,11 @@ if not CWD:
 # make it work with cxfreeze
 if os.path.isfile(CWD):
     CWD = os.path.dirname(CWD)
+    if CWD.endswith("/lib"):
+        CWD = CWD.rsplit("/lib",1)[0]
     CXFREEZE = True
+    if not WINDOWS:
+        os.environ['CARLA_MAGIC_FILE'] = os.path.join(CWD, "magic.mgc")
 else:
     CXFREEZE = False
 
@@ -620,8 +626,8 @@ def toList(value):
 # ------------------------------------------------------------------------------------------------------------
 # Get Icon from user theme, using our own as backup (Oxygen)
 
-def getIcon(icon, size = 16):
-    return QIcon.fromTheme(icon, QIcon(":/%ix%i/%s.png" % (size, size, icon)))
+def getIcon(icon, size, qrcformat):
+    return QIcon.fromTheme(icon, QIcon(":/%ix%i/%s.%s" % (size, size, icon, qrcformat)))
 
 # ------------------------------------------------------------------------------------------------------------
 # Handle some basic command-line arguments shared between all carla variants
@@ -664,13 +670,18 @@ def handleInitialCommandLineArguments(file):
             print(" and OPTION can be one or more of the following:")
             print("")
             print("    --cnprefix\t Set a prefix for client names in multi-client mode.")
-            print("    --gdb     \t Run Carla inside gdb.")
-            print(" -n,--no-gui  \t Run Carla headless, don't show UI.")
-            print("")
+            if isinstance(gCarla.nogui, bool):
+                print("    --gdb     \t Run Carla inside gdb.")
+                print(" -n,--no-gui  \t Run Carla headless, don't show UI.")
+                print("")
             print(" -h,--help    \t Print this help text and exit.")
             print(" -v,--version \t Print version information and exit.")
             print("")
 
+            if not isinstance(gCarla.nogui, bool):
+                print("NOTE: when using %s the FILE is only valid the first time the backend is started" % initName)
+                sys.exit(1)
+
             sys.exit(0)
 
         elif arg in ("-v", "--v", "-version", "--version"):
@@ -683,12 +694,20 @@ def handleInitialCommandLineArguments(file):
             print("  Binary dir:     %s" % pathBinaries)
             print("  Resources dir:  %s" % pathResources)
 
-            sys.exit(0)
+            sys.exit(1 if gCarla.nogui else 0)
 
         elif readPrefixNext:
             readPrefixNext = False
             gCarla.cnprefix = arg
 
+    if gCarla.nogui and not isinstance(gCarla.nogui, bool):
+        if os.fork():
+            # pylint: disable=protected-access
+            os._exit(0)
+            # pylint: enable=protected-access
+        else:
+            os.setsid()
+
     return (initName, libPrefix)
 
 # ------------------------------------------------------------------------------------------------------------
