@@ -1,6 +1,6 @@
 /*
  * JackBridge
- * Copyright (C) 2013-2015 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2013-2019 Filipe Coelho <falktx@falktx.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -43,6 +43,7 @@
 # include <jack/transport.h>
 # include <jack/session.h>
 # include <jack/metadata.h>
+# include <jack/uuid.h>
 #else
 
 #include <cstddef>
@@ -94,6 +95,9 @@
 #define JACK_POSITION_MASK (JackPositionBBT|JackPositionTimecode|JackBBTFrameOffset|JackAudioVideoRatio|JackVideoFrameOffset)
 #define EXTENDED_TIME_INFO
 
+#define JACK_UUID_SIZE 36
+#define JACK_UUID_STRING_SIZE (JACK_UUID_SIZE+1) /* includes trailing null */
+
 extern "C" {
 
 enum JackOptions {
@@ -132,7 +136,8 @@ enum JackPortFlags {
     JackPortIsOutput   = 0x02,
     JackPortIsPhysical = 0x04,
     JackPortCanMonitor = 0x08,
-    JackPortIsTerminal = 0x10
+    JackPortIsTerminal = 0x10,
+    JackPortIsControlVoltage = 0x100
 };
 
 enum JackTransportState {
@@ -293,9 +298,13 @@ JACKBRIDGE_API bool           jackbridge_client_close(jack_client_t* client);
 JACKBRIDGE_API int   jackbridge_client_name_size();
 JACKBRIDGE_API char* jackbridge_get_client_name(jack_client_t* client);
 
+JACKBRIDGE_API char* jackbridge_client_get_uuid(jack_client_t* client);
 JACKBRIDGE_API char* jackbridge_get_uuid_for_client_name(jack_client_t* client, const char* name);
 JACKBRIDGE_API char* jackbridge_get_client_name_by_uuid(jack_client_t* client, const char* uuid);
 
+JACKBRIDGE_API bool jackbridge_uuid_parse(const char* buf, jack_uuid_t* uuid);
+JACKBRIDGE_API void jackbridge_uuid_unparse(jack_uuid_t uuid, char buf[JACK_UUID_STRING_SIZE]);
+
 JACKBRIDGE_API bool jackbridge_activate(jack_client_t* client);
 JACKBRIDGE_API bool jackbridge_deactivate(jack_client_t* client);
 JACKBRIDGE_API bool jackbridge_is_realtime(jack_client_t* client);
