@@ -2,7 +2,7 @@
   ==============================================================================
 
    This file is part of the JUCE library.
-   Copyright (c) 2017 - ROLI Ltd.
+   Copyright (c) 2020 - Raw Material Software Limited
 
    JUCE is an open source library subject to commercial or open-source
    licensing.
@@ -23,99 +23,117 @@
 namespace juce
 {
 
-class ZipFile::ZipEntryHolder
+inline uint16 readUnalignedLittleEndianShort (const void* buffer)
 {
-public:
-    ZipEntryHolder (const char* const buffer, const int fileNameLen)
+    auto data = readUnaligned<uint16> (buffer);
+    return ByteOrder::littleEndianShort (&data);
+}
+
+inline uint32 readUnalignedLittleEndianInt (const void* buffer)
+{
+    auto data = readUnaligned<uint32> (buffer);
+    return ByteOrder::littleEndianInt (&data);
+}
+
+struct ZipFile::ZipEntryHolder
+{
+    ZipEntryHolder (const char* buffer, int fileNameLen)
     {
-        isCompressed            = ByteOrder::littleEndianShort (buffer + 10) != 0;
-        entry.fileTime          = parseFileTime ((uint32) ByteOrder::littleEndianShort (buffer + 12),
-                                                 (uint32) ByteOrder::littleEndianShort (buffer + 14));
-        compressedSize          = (int64) (uint32) ByteOrder::littleEndianInt (buffer + 20);
-        entry.uncompressedSize  = (int64) (uint32) ByteOrder::littleEndianInt (buffer + 24);
-        streamOffset            = (int64) (uint32) ByteOrder::littleEndianInt (buffer + 42);
-        entry.filename          = String::fromUTF8 (buffer + 46, fileNameLen);
+        isCompressed           = readUnalignedLittleEndianShort (buffer + 10) != 0;
+        entry.fileTime         = parseFileTime (readUnalignedLittleEndianShort (buffer + 12),
+                                                readUnalignedLittleEndianShort (buffer + 14));
+        compressedSize         = (int64) readUnalignedLittleEndianInt (buffer + 20);
+        entry.uncompressedSize = (int64) readUnalignedLittleEndianInt (buffer + 24);
+        streamOffset           = (int64) readUnalignedLittleEndianInt (buffer + 42);
+
+        entry.externalFileAttributes = readUnalignedLittleEndianInt (buffer + 38);
+        auto fileType = (entry.externalFileAttributes >> 28) & 0xf;
+        entry.isSymbolicLink = (fileType == 0xA);
+
+        entry.filename = String::fromUTF8 (buffer + 46, fileNameLen);
     }
 
-    struct FileNameComparator
+    static Time parseFileTime (uint32 time, uint32 date) noexcept
     {
-        static int compareElements (const ZipEntryHolder* e1, const ZipEntryHolder* e2) noexcept
-        {
-            return e1->entry.filename.compare (e2->entry.filename);
-        }
-    };
+        auto year      = (int) (1980 + (date >> 9));
+        auto month     = (int) (((date >> 5) & 15) - 1);
+        auto day       = (int) (date & 31);
+        auto hours     = (int) time >> 11;
+        auto minutes   = (int) ((time >> 5) & 63);
+        auto seconds   = (int) ((time & 31) << 1);
+
+        return { year, month, day, hours, minutes, seconds };
+    }
 
     ZipEntry entry;
     int64 streamOffset, compressedSize;
     bool isCompressed;
-
-private:
-    static Time parseFileTime (uint32 time, uint32 date) noexcept
-    {
-        const int year      = 1980 + (date >> 9);
-        const int month     = ((date >> 5) & 15) - 1;
-        const int day       = date & 31;
-        const int hours     = time >> 11;
-        const int minutes   = (time >> 5) & 63;
-        const int seconds   = (int) ((time & 31) << 1);
-
-        return Time (year, month, day, hours, minutes, seconds);
-    }
 };
 
 //==============================================================================
-namespace
+static int64 findCentralDirectoryFileHeader (InputStream& input, int& numEntries)
 {
-    int findEndOfZipEntryTable (InputStream& input, int& numEntries)
+    BufferedInputStream in (input, 8192);
+
+    in.setPosition (in.getTotalLength());
+    auto pos = in.getPosition();
+    auto lowestPos = jmax ((int64) 0, pos - 1048576);
+    char buffer[32] = {};
+
+    while (pos > lowestPos)
     {
-        BufferedInputStream in (input, 8192);
+        in.setPosition (pos - 22);
+        pos = in.getPosition();
+        memcpy (buffer + 22, buffer, 4);
 
-        in.setPosition (in.getTotalLength());
-        int64 pos = in.getPosition();
-        const int64 lowestPos = jmax ((int64) 0, pos - 1024);
+        if (in.read (buffer, 22) != 22)
+            return 0;
 
-        char buffer [32] = { 0 };
-
-        while (pos > lowestPos)
+        for (int i = 0; i < 22; ++i)
         {
-            in.setPosition (pos - 22);
-            pos = in.getPosition();
-            memcpy (buffer + 22, buffer, 4);
-
-            if (in.read (buffer, 22) != 22)
-                return 0;
-
-            for (int i = 0; i < 22; ++i)
+            if (readUnalignedLittleEndianInt (buffer + i) == 0x06054b50)
             {
-                if (ByteOrder::littleEndianInt (buffer + i) == 0x06054b50)
-                {
-                    in.setPosition (pos + i);
-                    in.read (buffer, 22);
-                    numEntries = ByteOrder::littleEndianShort (buffer + 10);
+                in.setPosition (pos + i);
+                in.read (buffer, 22);
+                numEntries = readUnalignedLittleEndianShort (buffer + 10);
+                auto offset = (int64) readUnalignedLittleEndianInt (buffer + 16);
 
-                    return (int) ByteOrder::littleEndianInt (buffer + 16);
+                if (offset >= 4)
+                {
+                    in.setPosition (offset);
+
+                    // This is a workaround for some zip files which seem to contain the
+                    // wrong offset for the central directory - instead of including the
+                    // header, they point to the byte immediately after it.
+                    if (in.readInt() != 0x02014b50)
+                    {
+                        in.setPosition (offset - 4);
+
+                        if (in.readInt() == 0x02014b50)
+                            offset -= 4;
+                    }
                 }
+
+                return offset;
             }
         }
-
-        return 0;
     }
+
+    return 0;
 }
 
 //==============================================================================
-class ZipFile::ZipInputStream  : public InputStream
+struct ZipFile::ZipInputStream  : public InputStream
 {
-public:
-    ZipInputStream (ZipFile& zf, ZipFile::ZipEntryHolder& zei)
+    ZipInputStream (ZipFile& zf, const ZipFile::ZipEntryHolder& zei)
         : file (zf),
           zipEntryHolder (zei),
-          pos (0),
-          headerSize (0),
           inputStream (zf.inputStream)
     {
         if (zf.inputSource != nullptr)
         {
-            inputStream = streamToDelete = file.inputSource->createInputStream();
+            streamToDelete.reset (file.inputSource->createInputStream());
+            inputStream = streamToDelete.get();
         }
         else
         {
@@ -124,7 +142,7 @@ public:
            #endif
         }
 
-        char buffer [30];
+        char buffer[30];
 
         if (inputStream != nullptr
              && inputStream->setPosition (zei.streamOffset)
@@ -136,7 +154,7 @@ public:
         }
     }
 
-    ~ZipInputStream()
+    ~ZipInputStream() override
     {
        #if JUCE_DEBUG
         if (inputStream != nullptr && inputStream == file.inputStream)
@@ -196,41 +214,36 @@ public:
 private:
     ZipFile& file;
     ZipEntryHolder zipEntryHolder;
-    int64 pos;
-    int headerSize;
+    int64 pos = 0;
+    int headerSize = 0;
     InputStream* inputStream;
-    ScopedPointer<InputStream> streamToDelete;
+    std::unique_ptr<InputStream> streamToDelete;
 
     JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ZipInputStream)
 };
 
 
 //==============================================================================
-ZipFile::ZipFile (InputStream* const stream, const bool deleteStreamWhenDestroyed)
+ZipFile::ZipFile (InputStream* stream, bool deleteStreamWhenDestroyed)
    : inputStream (stream)
 {
     if (deleteStreamWhenDestroyed)
-        streamToDelete = inputStream;
+        streamToDelete.reset (inputStream);
 
     init();
 }
 
-ZipFile::ZipFile (InputStream& stream)
-   : inputStream (&stream)
+ZipFile::ZipFile (InputStream& stream)  : inputStream (&stream)
 {
     init();
 }
 
-ZipFile::ZipFile (const File& file)
-    : inputStream (nullptr),
-      inputSource (new FileInputSource (file))
+ZipFile::ZipFile (const File& file)  : inputSource (new FileInputSource (file))
 {
     init();
 }
 
-ZipFile::ZipFile (InputSource* const source)
-    : inputStream (nullptr),
-      inputSource (source)
+ZipFile::ZipFile (InputSource* source)  : inputSource (source)
 {
     init();
 }
@@ -260,7 +273,7 @@ int ZipFile::getNumEntries() const noexcept
 
 const ZipFile::ZipEntry* ZipFile::getEntry (const int index) const noexcept
 {
-    if (ZipEntryHolder* const zei = entries [index])
+    if (auto* zei = entries[index])
         return &(zei->entry);
 
     return nullptr;
@@ -289,7 +302,7 @@ InputStream* ZipFile::createStreamForEntry (const int index)
 {
     InputStream* stream = nullptr;
 
-    if (ZipEntryHolder* const zei = entries[index])
+    if (auto* zei = entries[index])
     {
         stream = new ZipInputStream (*this, *zei);
 
@@ -318,55 +331,54 @@ InputStream* ZipFile::createStreamForEntry (const ZipEntry& entry)
 
 void ZipFile::sortEntriesByFilename()
 {
-    ZipEntryHolder::FileNameComparator sorter;
-    entries.sort (sorter);
+    std::sort (entries.begin(), entries.end(),
+               [] (const ZipEntryHolder* e1, const ZipEntryHolder* e2) { return e1->entry.filename < e2->entry.filename; });
 }
 
 //==============================================================================
 void ZipFile::init()
 {
-    ScopedPointer<InputStream> toDelete;
+    std::unique_ptr<InputStream> toDelete;
     InputStream* in = inputStream;
 
     if (inputSource != nullptr)
     {
         in = inputSource->createInputStream();
-        toDelete = in;
+        toDelete.reset (in);
     }
 
     if (in != nullptr)
     {
         int numEntries = 0;
-        int pos = findEndOfZipEntryTable (*in, numEntries);
+        auto centralDirectoryPos = findCentralDirectoryFileHeader (*in, numEntries);
 
-        if (pos >= 0 && pos < in->getTotalLength())
+        if (centralDirectoryPos >= 0 && centralDirectoryPos < in->getTotalLength())
         {
-            const int size = (int) (in->getTotalLength() - pos);
+            auto size = (size_t) (in->getTotalLength() - centralDirectoryPos);
 
-            in->setPosition (pos);
+            in->setPosition (centralDirectoryPos);
             MemoryBlock headerData;
 
-            if (in->readIntoMemoryBlock (headerData, size) == (size_t) size)
+            if (in->readIntoMemoryBlock (headerData, (ssize_t) size) == size)
             {
-                pos = 0;
+                size_t pos = 0;
 
                 for (int i = 0; i < numEntries; ++i)
                 {
                     if (pos + 46 > size)
                         break;
 
-                    const char* const buffer = static_cast<const char*> (headerData.getData()) + pos;
-
-                    const int fileNameLen = ByteOrder::littleEndianShort (buffer + 28);
+                    auto* buffer = static_cast<const char*> (headerData.getData()) + pos;
+                    auto fileNameLen = readUnalignedLittleEndianShort (buffer + 28u);
 
                     if (pos + 46 + fileNameLen > size)
                         break;
 
                     entries.add (new ZipEntryHolder (buffer, fileNameLen));
 
-                    pos += 46 + fileNameLen
-                            + ByteOrder::littleEndianShort (buffer + 30)
-                            + ByteOrder::littleEndianShort (buffer + 32);
+                    pos += 46u + fileNameLen
+                            + readUnalignedLittleEndianShort (buffer + 30u)
+                            + readUnalignedLittleEndianShort (buffer + 32u);
                 }
             }
         }
@@ -378,7 +390,8 @@ Result ZipFile::uncompressTo (const File& targetDirectory,
 {
     for (int i = 0; i < entries.size(); ++i)
     {
-        Result result (uncompressEntry (i, targetDirectory, shouldOverwriteFiles));
+        auto result = uncompressEntry (i, targetDirectory, shouldOverwriteFiles);
+
         if (result.failed())
             return result;
     }
@@ -386,24 +399,25 @@ Result ZipFile::uncompressTo (const File& targetDirectory,
     return Result::ok();
 }
 
-Result ZipFile::uncompressEntry (const int index,
-                                 const File& targetDirectory,
-                                 bool shouldOverwriteFiles)
+Result ZipFile::uncompressEntry (int index, const File& targetDirectory, bool shouldOverwriteFiles)
 {
-    const ZipEntryHolder* zei = entries.getUnchecked (index);
+    auto* zei = entries.getUnchecked (index);
 
    #if JUCE_WINDOWS
-    const String entryPath (zei->entry.filename);
+    auto entryPath = zei->entry.filename;
    #else
-    const String entryPath (zei->entry.filename.replaceCharacter ('\\', '/'));
+    auto entryPath = zei->entry.filename.replaceCharacter ('\\', '/');
    #endif
 
-    const File targetFile (targetDirectory.getChildFile (entryPath));
+    if (entryPath.isEmpty())
+        return Result::ok();
+
+    auto targetFile = targetDirectory.getChildFile (entryPath);
 
     if (entryPath.endsWithChar ('/') || entryPath.endsWithChar ('\\'))
         return targetFile.createDirectory(); // (entry is a directory, not a file)
 
-    ScopedPointer<InputStream> in (createStreamForEntry (index));
+    std::unique_ptr<InputStream> in (createStreamForEntry (index));
 
     if (in == nullptr)
         return Result::fail ("Failed to open the zip file for reading");
@@ -420,6 +434,15 @@ Result ZipFile::uncompressEntry (const int index,
     if (! targetFile.getParentDirectory().createDirectory())
         return Result::fail ("Failed to create target folder: " + targetFile.getParentDirectory().getFullPathName());
 
+    if (zei->entry.isSymbolicLink)
+    {
+        String originalFilePath (in->readEntireStreamAsString()
+                                    .replaceCharacter (L'/', File::getSeparatorChar()));
+
+        if (! File::createSymbolicLink (targetFile, originalFilePath, true))
+            return Result::fail ("Failed to create symbolic link: " + originalFilePath);
+    }
+    else
     {
         FileOutputStream out (targetFile);
 
@@ -438,23 +461,30 @@ Result ZipFile::uncompressEntry (const int index,
 
 
 //==============================================================================
-class ZipFile::Builder::Item
+struct ZipFile::Builder::Item
 {
-public:
     Item (const File& f, InputStream* s, int compression, const String& storedPath, Time time)
-        : file (f), stream (s), storedPathname (storedPath), fileTime (time),
-          compressedSize (0), uncompressedSize (0), headerStart (0),
-          compressionLevel (compression), checksum (0)
+        : file (f), stream (s), storedPathname (storedPath), fileTime (time), compressionLevel (compression)
     {
+        symbolicLink = (file.exists() && file.isSymbolicLink());
     }
 
     bool writeData (OutputStream& target, const int64 overallStartPosition)
     {
         MemoryOutputStream compressedData ((size_t) file.getSize());
 
-        if (compressionLevel > 0)
+        if (symbolicLink)
         {
-            GZIPCompressorOutputStream compressor (&compressedData, compressionLevel, false,
+            auto relativePath = file.getNativeLinkedTarget().replaceCharacter (File::getSeparatorChar(), L'/');
+
+            uncompressedSize = relativePath.length();
+
+            checksum = zlibNamespace::crc32 (0, (uint8_t*) relativePath.toRawUTF8(), (unsigned int) uncompressedSize);
+            compressedData << relativePath;
+        }
+        else if (compressionLevel > 0)
+        {
+            GZIPCompressorOutputStream compressor (compressedData, compressionLevel,
                                                    GZIPCompressorOutputStream::windowBitsRaw);
             if (! writeSource (compressor))
                 return false;
@@ -479,12 +509,12 @@ public:
     bool writeDirectoryEntry (OutputStream& target)
     {
         target.writeInt (0x02014b50);
-        target.writeShort (20); // version written
+        target.writeShort (symbolicLink ? 0x0314 : 0x0014);
         writeFlagsAndSizes (target);
         target.writeShort (0); // comment length
         target.writeShort (0); // start disk num
         target.writeShort (0); // internal attributes
-        target.writeInt (0); // external attributes
+        target.writeInt ((int) (symbolicLink ? 0xA1ED0000 : 0)); // external attributes
         target.writeInt ((int) (uint32) headerStart);
         target << storedPathname;
 
@@ -493,12 +523,13 @@ public:
 
 private:
     const File file;
-    ScopedPointer<InputStream> stream;
+    std::unique_ptr<InputStream> stream;
     String storedPathname;
     Time fileTime;
-    int64 compressedSize, uncompressedSize, headerStart;
-    int compressionLevel;
-    unsigned long checksum;
+    int64 compressedSize = 0, uncompressedSize = 0, headerStart = 0;
+    int compressionLevel = 0;
+    unsigned long checksum = 0;
+    bool symbolicLink = false;
 
     static void writeTimeAndDate (OutputStream& target, Time t)
     {
@@ -523,7 +554,7 @@ private:
 
         while (! stream->isExhausted())
         {
-            const int bytesRead = stream->read (buffer, bufferSize);
+            auto bytesRead = stream->read (buffer, bufferSize);
 
             if (bytesRead < 0)
                 return false;
@@ -533,7 +564,7 @@ private:
             uncompressedSize += bytesRead;
         }
 
-        stream = nullptr;
+        stream.reset();
         return true;
     }
 
@@ -541,12 +572,12 @@ private:
     {
         target.writeShort (10); // version needed
         target.writeShort ((short) (1 << 11)); // this flag indicates UTF-8 filename encoding
-        target.writeShort (compressionLevel > 0 ? (short) 8 : (short) 0);
+        target.writeShort ((! symbolicLink && compressionLevel > 0) ? (short) 8 : (short) 0); //symlink target path is not compressed
         writeTimeAndDate (target, fileTime);
         target.writeInt ((int) checksum);
         target.writeInt ((int) (uint32) compressedSize);
         target.writeInt ((int) (uint32) uncompressedSize);
-        target.writeShort ((short) storedPathname.toUTF8().sizeInBytes() - 1);
+        target.writeShort (static_cast<short> (storedPathname.toUTF8().sizeInBytes() - 1));
         target.writeShort (0); // extra field length
     }
 
@@ -557,7 +588,7 @@ private:
 ZipFile::Builder::Builder() {}
 ZipFile::Builder::~Builder() {}
 
-void ZipFile::Builder::addFile (const File& file, const int compression, const String& path)
+void ZipFile::Builder::addFile (const File& file, int compression, const String& path)
 {
     items.add (new Item (file, nullptr, compression,
                          path.isEmpty() ? file.getFileName() : path,
@@ -568,12 +599,12 @@ void ZipFile::Builder::addEntry (InputStream* stream, int compression, const Str
 {
     jassert (stream != nullptr); // must not be null!
     jassert (path.isNotEmpty());
-    items.add (new Item (File(), stream, compression, path, time));
+    items.add (new Item ({}, stream, compression, path, time));
 }
 
 bool ZipFile::Builder::writeToStream (OutputStream& target, double* const progress) const
 {
-    const int64 fileStart = target.getPosition();
+    auto fileStart = target.getPosition();
 
     for (int i = 0; i < items.size(); ++i)
     {
@@ -584,13 +615,13 @@ bool ZipFile::Builder::writeToStream (OutputStream& target, double* const progre
             return false;
     }
 
-    const int64 directoryStart = target.getPosition();
+    auto directoryStart = target.getPosition();
 
-    for (int i = 0; i < items.size(); ++i)
-        if (! items.getUnchecked (i)->writeDirectoryEntry (target))
+    for (auto* item : items)
+        if (! item->writeDirectoryEntry (target))
             return false;
 
-    const int64 directoryEnd = target.getPosition();
+    auto directoryEnd = target.getPosition();
 
     target.writeInt (0x06054b50);
     target.writeShort (0);
@@ -607,4 +638,54 @@ bool ZipFile::Builder::writeToStream (OutputStream& target, double* const progre
     return true;
 }
 
+
+//==============================================================================
+//==============================================================================
+#if JUCE_UNIT_TESTS
+
+struct ZIPTests   : public UnitTest
+{
+    ZIPTests()
+        : UnitTest ("ZIP", UnitTestCategories::compression)
+    {}
+
+    void runTest() override
+    {
+        beginTest ("ZIP");
+
+        ZipFile::Builder builder;
+        StringArray entryNames { "first", "second", "third" };
+        HashMap<String, MemoryBlock> blocks;
+
+        for (auto& entryName : entryNames)
+        {
+            auto& block = blocks.getReference (entryName);
+            MemoryOutputStream mo (block, false);
+            mo << entryName;
+            mo.flush();
+            builder.addEntry (new MemoryInputStream (block, false), 9, entryName, Time::getCurrentTime());
+        }
+
+        MemoryBlock data;
+        MemoryOutputStream mo (data, false);
+        builder.writeToStream (mo, nullptr);
+        MemoryInputStream mi (data, false);
+
+        ZipFile zip (mi);
+
+        expectEquals (zip.getNumEntries(), entryNames.size());
+
+        for (auto& entryName : entryNames)
+        {
+            auto* entry = zip.getEntry (entryName);
+            std::unique_ptr<InputStream> input (zip.createStreamForEntry (*entry));
+            expectEquals (input->readEntireStreamAsString(), entryName);
+        }
+    }
+};
+
+static ZIPTests zipTests;
+
+#endif
+
 } // namespace juce
