@@ -1109,6 +1109,11 @@ File File::getCurrentWorkingDirectory()
     return File (String (dest));
 }
 
+bool File::setAsCurrentWorkingDirectory() const
+{
+    return SetCurrentDirectory (getFullPathName().toUTF8()) != FALSE;
+}
+
 bool File::isSymbolicLink() const
 {
     return (GetFileAttributes (fullPath.toUTF8()) & FILE_ATTRIBUTE_REPARSE_POINT) != 0;
@@ -1342,6 +1347,11 @@ File File::getCurrentWorkingDirectory()
     return File (CharPointer_UTF8 (cwd));
 }
 
+bool File::setAsCurrentWorkingDirectory() const
+{
+    return chdir (getFullPathName().toUTF8()) == 0;
+}
+
 File water_getExecutableFile();
 File water_getExecutableFile()
 {
