@@ -34,6 +34,19 @@ void Lv2PluginBaseClass<NativeTimeInfo>::clearTimeData() noexcept
     carla_zeroStruct(fTimeInfo);
 }
 
+struct PreviewData {
+    char type;
+    uint32_t size;
+    const void* buffer;
+    bool shouldSend;
+
+    PreviewData()
+        : type(0),
+          size(0),
+          buffer(nullptr),
+          shouldSend(false) {}
+};
+
 // --------------------------------------------------------------------------------------------------------------------
 // Carla Internal Plugin API exposed as LV2 plugin
 
@@ -54,9 +67,13 @@ public:
           fProgramDesc({0, 0, nullptr}),
 #endif
           kIgnoreParameters(std::strncmp(desc->label, "carla", 5) == 0),
+          fAtomForge(),
           fMidiEventCount(0),
           fLastProjectPath(),
           fLoadedFile(),
+          fPreviewData(),
+          fNeedsNotifyFileChanged(false),
+          fPluginNeedsIdle(0),
           fWorkerUISignal(0)
     {
         carla_zeroStruct(fHost);
@@ -88,6 +105,9 @@ public:
         fHost.ui_open_file           = host_ui_open_file;
         fHost.ui_save_file           = host_ui_save_file;
         fHost.dispatcher             = host_dispatcher;
+
+        carla_zeroStruct(fAtomForge);
+        lv2_atom_forge_init(&fAtomForge, fUridMap);
     }
 
     ~NativePlugin()
@@ -234,7 +254,7 @@ public:
                     if (event == nullptr)
                         continue;
 
-                    if (event->body.type == fURIs.uiEvents && fWorkerUISignal != -1)
+                    if (event->body.type == fURIs.carlaUiEvents && fWorkerUISignal != -1)
                     {
                         CARLA_SAFE_ASSERT_CONTINUE((fDescriptor->hints & NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE) == 0);
 
@@ -258,7 +278,8 @@ public:
                     {
                         const LV2_Atom_Object* const obj = (const LV2_Atom_Object*)(&event->body);
 
-                        if (obj->body.otype == fURIs.patchSet) {
+                        if (obj->body.otype == fURIs.patchSet)
+                        {
                             // Get property URI.
                             const LV2_Atom* property = nullptr;
                             lv2_atom_object_get(obj, fURIs.patchProperty, &property, 0);
@@ -287,12 +308,22 @@ public:
                                                    static_cast<uint32_t>(std::strlen(filepath) + 1U),
                                                    filepath);
                         }
-
+                        else if (obj->body.otype == fURIs.patchGet)
+                        {
+                            if (fDescriptor->hints & NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE)
+                                fNeedsNotifyFileChanged = true;
+                            if (fPreviewData.buffer != nullptr)
+                                fPreviewData.shouldSend = true;
+                        }
                         continue;
                     }
 
                     if (event->body.type != fURIs.midiEvent)
                         continue;
+
+                    // anything past this point assumes plugin with MIDI input
+                    CARLA_SAFE_ASSERT_CONTINUE(fPorts.numMidiIns > 0);
+
                     if (event->body.size > 4)
                         continue;
                     if (event->time.frames >= frames)
@@ -316,10 +347,109 @@ public:
                         break;
                 }
             }
+
+            if (fNeedsNotifyFileChanged || fPreviewData.shouldSend)
+            {
+                uint8_t atomBuf[4096];
+                LV2_Atom_Forge atomForge = fAtomForge;
+                lv2_atom_forge_set_buffer(&atomForge, atomBuf, sizeof(atomBuf));
+
+                const int numEvents = fNeedsNotifyFileChanged && fPreviewData.shouldSend ? 2 : 1;
+
+                if (fNeedsNotifyFileChanged)
+                {
+                    fNeedsNotifyFileChanged = false;
+
+                    LV2_Atom_Forge_Frame forgeFrame;
+                    lv2_atom_forge_object(&atomForge, &forgeFrame, 0, fURIs.patchSet);
+
+                    lv2_atom_forge_key(&atomForge, fURIs.patchProperty);
+
+                    /*  */ if (std::strcmp(fDescriptor->label, "audiofile") == 0) {
+                        lv2_atom_forge_urid(&atomForge, fURIs.carlaFileAudio);
+                    } else if (std::strcmp(fDescriptor->label, "midifile") == 0) {
+                        lv2_atom_forge_urid(&atomForge, fURIs.carlaFileMIDI);
+                    } else {
+                        lv2_atom_forge_urid(&atomForge, fURIs.carlaFile);
+                    }
+
+                    lv2_atom_forge_key(&atomForge, fURIs.patchValue);
+                    lv2_atom_forge_path(&atomForge,
+                                        fLoadedFile.buffer(),
+                                        static_cast<uint32_t>(fLoadedFile.length()+1));
+
+                    lv2_atom_forge_pop(&atomForge, &forgeFrame);
+                }
+
+                if (fPreviewData.shouldSend)
+                {
+                    const char ptype = fPreviewData.type;
+                    const uint32_t psize = fPreviewData.size;
+                    const void* const pbuffer = fPreviewData.buffer;
+                    fPreviewData.shouldSend = false;
+
+                    LV2_Atom_Forge_Frame forgeFrame;
+                    lv2_atom_forge_object(&atomForge, &forgeFrame, 0, fURIs.patchSet);
+
+                    lv2_atom_forge_key(&atomForge, fURIs.patchProperty);
+                    lv2_atom_forge_urid(&atomForge, fURIs.carlaPreview);
+
+                    lv2_atom_forge_key(&atomForge, fURIs.patchValue);
+
+                    switch (ptype)
+                    {
+                    case 'b':
+                        lv2_atom_forge_vector(&atomForge, sizeof(int32_t), fURIs.atomBool, psize, pbuffer);
+                        break;
+                    case 'i':
+                        lv2_atom_forge_vector(&atomForge, sizeof(int32_t), fURIs.atomInt, psize, pbuffer);
+                        break;
+                    case 'f':
+                        lv2_atom_forge_vector(&atomForge, sizeof(float), fURIs.atomFloat, psize, pbuffer);
+                        break;
+                    default:
+                        carla_stderr2("Preview data buffer has wrong type '%c' (and size %u)", ptype, psize);
+                        break;
+                    }
+
+                    lv2_atom_forge_pop(&atomForge, &forgeFrame);
+                }
+
+                LV2_Atom* atom = (LV2_Atom*)atomBuf;
+                LV2_Atom_Sequence* const seq = fPorts.eventsOut[0];
+                Ports::EventsOutData& mData(fPorts.eventsOutData[0]);
+
+                for (int i=0; i<numEvents; ++i)
+                {
+                    if (sizeof(LV2_Atom_Event) + atom->size <= mData.capacity - mData.offset)
+                    {
+                        LV2_Atom_Event* const aev = (LV2_Atom_Event*)(LV2_ATOM_CONTENTS(LV2_Atom_Sequence, seq) + mData.offset);
+
+                        aev->time.frames = 0;
+                        aev->body.size   = atom->size;
+                        aev->body.type   = atom->type;
+                        std::memcpy(LV2_ATOM_BODY(&aev->body), atom + 1, atom->size);
+
+                        const uint32_t size = lv2_atom_pad_size(static_cast<uint32_t>(sizeof(LV2_Atom_Event) + atom->size));
+                        mData.offset       += size;
+                        seq->atom.size     += size;
+                    }
+
+                    atom = (LV2_Atom*)(atomBuf + lv2_atom_total_size(atom));
+                }
+            }
         }
 
         fDescriptor->process(fHandle, fPorts.audioCVIns, fPorts.audioCVOuts, frames, fMidiEvents, fMidiEventCount);
 
+        if (fPluginNeedsIdle == 1)
+        {
+            fPluginNeedsIdle = 2;
+            const char* const msg = "_idle_";
+            const size_t msgSize = std::strlen(msg);
+            fWorker->schedule_work(fWorker->handle, static_cast<uint32_t>(msgSize + 1U), msg);
+        }
+
         if (fWorkerUISignal == -1 && fPorts.hasUI)
         {
             const char* const msg = "quit";
@@ -334,7 +464,7 @@ public:
 
                 aev->time.frames = 0;
                 aev->body.size   = msgSize;
-                aev->body.type   = fURIs.uiEvents;
+                aev->body.type   = fURIs.carlaUiEvents;
                 std::memcpy(LV2_ATOM_BODY(&aev->body), msg, msgSize);
 
                 const uint32_t size = lv2_atom_pad_size(static_cast<uint32_t>(sizeof(LV2_Atom_Event) + msgSize));
@@ -444,22 +574,57 @@ public:
 
         if (fDescriptor->hints & NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE)
         {
+            if (fLoadedFile.isEmpty())
+                return LV2_STATE_SUCCESS;
+
+            const LV2_State_Free_Path* freePath = nullptr;
+            const LV2_State_Map_Path* mapPath = nullptr;
+
+            if (features != nullptr)
+            {
+                for (int i=0; features[i] != nullptr; ++i)
+                {
+                    /**/ if (freePath == nullptr && std::strcmp(features[i]->URI, LV2_STATE__freePath) == 0)
+                        freePath = (const LV2_State_Free_Path*)features[i]->data;
+                    else if (mapPath == nullptr && std::strcmp(features[i]->URI, LV2_STATE__mapPath) == 0)
+                        mapPath = (const LV2_State_Map_Path*)features[i]->data;
+                }
+            }
+
+            if (mapPath == nullptr || mapPath->abstract_path == nullptr)
+                return LV2_STATE_ERR_NO_FEATURE;
+
+            char* path = mapPath->abstract_path(mapPath->handle, fLoadedFile.buffer());
+
             store(handle,
                   fUridMap->map(fUridMap->handle, "http://kxstudio.sf.net/ns/carla/file"),
-                  fLoadedFile.buffer(),
-                  fLoadedFile.length()+1,
+                  path,
+                  std::strlen(path)+1,
                   fURIs.atomPath,
-                  LV2_STATE_IS_POD);
+                  LV2_STATE_IS_POD|LV2_STATE_IS_PORTABLE);
+
+            if (freePath != nullptr && freePath->free_path != nullptr)
+                freePath->free_path(freePath->handle, path);
+#ifndef CARLA_OS_WIN
+            // this is not safe to call under windows
+            else
+                std::free(path);
+#endif
+
             return LV2_STATE_SUCCESS;
         }
 
         if ((fDescriptor->hints & NATIVE_PLUGIN_USES_STATE) == 0 || fDescriptor->get_state == nullptr)
-            return LV2_STATE_ERR_NO_FEATURE;
+            return LV2_STATE_ERR_UNKNOWN;
 
         if (char* const state = fDescriptor->get_state(fHandle))
         {
-            store(handle, fUridMap->map(fUridMap->handle, "http://kxstudio.sf.net/ns/carla/chunk"),
-                  state, std::strlen(state)+1, fURIs.atomString, LV2_STATE_IS_POD|LV2_STATE_IS_PORTABLE);
+            store(handle,
+                  fUridMap->map(fUridMap->handle, "http://kxstudio.sf.net/ns/carla/chunk"),
+                  state,
+                  std::strlen(state)+1,
+                  fURIs.atomString,
+                  LV2_STATE_IS_POD|LV2_STATE_IS_PORTABLE);
             std::free(state);
             return LV2_STATE_SUCCESS;
         }
@@ -467,8 +632,10 @@ public:
         return LV2_STATE_ERR_UNKNOWN;
     }
 
-    LV2_State_Status lv2_restore(const LV2_State_Retrieve_Function retrieve, const LV2_State_Handle handle,
-                                 uint32_t flags, const LV2_Feature* const* const features)
+    LV2_State_Status lv2_restore(const LV2_State_Retrieve_Function retrieve,
+                                 const LV2_State_Handle handle,
+                                 uint32_t flags,
+                                 const LV2_Feature* const* const features)
     {
         saveLastProjectPathIfPossible(features);
 
@@ -481,21 +648,53 @@ public:
             const void* const data = retrieve(handle,
                                               fUridMap->map(fUridMap->handle, "http://kxstudio.sf.net/ns/carla/file"),
                                               &size, &type, &flags);
+            if (size <= 1 || type == 0)
+                return LV2_STATE_ERR_NO_PROPERTY;
 
             CARLA_SAFE_ASSERT_RETURN(type == fURIs.atomPath, LV2_STATE_ERR_UNKNOWN);
 
+            const LV2_State_Free_Path* freePath = nullptr;
+            const LV2_State_Map_Path* mapPath = nullptr;
+
+            if (features != nullptr)
+            {
+                for (int i=0; features[i] != nullptr; ++i)
+                {
+                    /**/ if (freePath == nullptr && std::strcmp(features[i]->URI, LV2_STATE__freePath) == 0)
+                        freePath = (const LV2_State_Free_Path*)features[i]->data;
+                    else if (mapPath == nullptr && std::strcmp(features[i]->URI, LV2_STATE__mapPath) == 0)
+                        mapPath = (const LV2_State_Map_Path*)features[i]->data;
+                }
+            }
+
+            if (mapPath == nullptr || mapPath->absolute_path == nullptr)
+                return LV2_STATE_ERR_NO_FEATURE;
+
             const char* const filename = (const char*)data;
 
-            fLoadedFile = filename;
-            fDescriptor->set_custom_data(fHandle, "file", filename);
+            char* const absolute_filename = mapPath->absolute_path(mapPath->handle, filename);
+            fLoadedFile = absolute_filename;
+
+            if (freePath != nullptr && freePath->free_path != nullptr)
+                freePath->free_path(freePath->handle, absolute_filename);
+#ifndef CARLA_OS_WIN
+            // this is not safe to call under windows
+            else
+                std::free(absolute_filename);
+#endif
+
+            fNeedsNotifyFileChanged = true;
+            fDescriptor->set_custom_data(fHandle, "file", fLoadedFile);
             return LV2_STATE_SUCCESS;
         }
 
         if ((fDescriptor->hints & NATIVE_PLUGIN_USES_STATE) == 0 || fDescriptor->set_state == nullptr)
-            return LV2_STATE_ERR_NO_FEATURE;
+            return LV2_STATE_ERR_UNKNOWN;
 
         size = type = 0;
-        const void* const data = retrieve(handle, fUridMap->map(fUridMap->handle, "http://kxstudio.sf.net/ns/carla/chunk"), &size, &type, &flags);
+        const void* const data = retrieve(handle,
+                                          fUridMap->map(fUridMap->handle, "http://kxstudio.sf.net/ns/carla/chunk"),
+                                          &size, &type, &flags);
 
         if (size == 0)
             return LV2_STATE_ERR_UNKNOWN;
@@ -517,6 +716,17 @@ public:
     {
         const char* const msg = (const char*)data;
 
+        if (std::strcmp(msg, "_idle_") == 0)
+        {
+            if (fDescriptor->hints & NATIVE_PLUGIN_REQUESTS_IDLE)
+            {
+                fPluginNeedsIdle = 0;
+                fDescriptor->dispatcher(fHandle, NATIVE_PLUGIN_OPCODE_IDLE, 0, 0, nullptr, 0.0f);
+                return LV2_WORKER_SUCCESS;
+            }
+            return LV2_WORKER_ERR_UNKNOWN;
+        }
+
         if (fDescriptor->hints & NATIVE_PLUGIN_NEEDS_UI_OPEN_SAVE)
         {
             fLoadedFile = msg;
@@ -582,6 +792,19 @@ public:
         return LV2_WORKER_SUCCESS;
     }
 
+    const LV2_Inline_Display_Image_Surface* lv2_idisp_render(const uint32_t width, const uint32_t height)
+    {
+        CARLA_SAFE_ASSERT_RETURN(fDescriptor->hints & NATIVE_PLUGIN_HAS_INLINE_DISPLAY, nullptr);
+        CARLA_SAFE_ASSERT_RETURN(fDescriptor->render_inline_display, nullptr);
+        CARLA_SAFE_ASSERT_RETURN(width > 0, nullptr);
+        CARLA_SAFE_ASSERT_RETURN(height > 0, nullptr);
+
+        const NativeInlineDisplayImageSurface* nsur = fDescriptor->render_inline_display(fHandle, width, height);
+        CARLA_SAFE_ASSERT_RETURN(nsur != nullptr, nullptr);
+
+        return (const LV2_Inline_Display_Image_Surface*)(nsur);
+    }
+
     // ----------------------------------------------------------------------------------------------------------------
 
     void lv2ui_instantiate(LV2UI_Write_Function writeFunction, LV2UI_Controller controller,
@@ -780,6 +1003,14 @@ protected:
         // nothing here
     }
 
+    void handlePreviewBufferData(const char type, const uint32_t size, const void* const buffer) noexcept
+    {
+        fPreviewData.type = type;
+        fPreviewData.size = size;
+        fPreviewData.buffer = buffer;
+        fPreviewData.shouldSend = true;
+    }
+
     void handleUiCustomDataChanged(const char* const key, const char* const value) const
     {
         carla_stdout("TODO: handleUiCustomDataChanged %s %s", key, value);
@@ -839,11 +1070,26 @@ protected:
         case NATIVE_HOST_OPCODE_RELOAD_ALL:
         case NATIVE_HOST_OPCODE_HOST_IDLE:
         case NATIVE_HOST_OPCODE_INTERNAL_PLUGIN:
-        case NATIVE_HOST_OPCODE_QUEUE_INLINE_DISPLAY:
-        case NATIVE_HOST_OPCODE_REQUEST_IDLE:
             // nothing
             break;
 
+        case NATIVE_HOST_OPCODE_REQUEST_IDLE:
+            CARLA_SAFE_ASSERT_RETURN(fDescriptor->hints & NATIVE_PLUGIN_REQUESTS_IDLE, 0);
+            if (fWorker != nullptr && fPluginNeedsIdle == 0)
+            {
+                fPluginNeedsIdle = 1;
+                return 1;
+            }
+            return 0;
+
+        case NATIVE_HOST_OPCODE_QUEUE_INLINE_DISPLAY:
+            if (fInlineDisplay != nullptr && fInlineDisplay->queue_draw != nullptr)
+            {
+                fInlineDisplay->queue_draw(fInlineDisplay->handle);
+                return 1;
+            }
+            return 0;
+
         case NATIVE_HOST_OPCODE_GET_FILE_PATH:
             CARLA_SAFE_ASSERT_RETURN(ptr != nullptr, 0);
             if (std::strcmp((char*)ptr, "carla") == 0 && fLastProjectPath != nullptr)
@@ -864,14 +1110,20 @@ protected:
             CARLA_SAFE_ASSERT_RETURN(value > 0, 0);
             handleUiResize(static_cast<uint32_t>(index), static_cast<uint32_t>(value));
             break;
+
+        case NATIVE_HOST_OPCODE_PREVIEW_BUFFER_DATA:
+            CARLA_SAFE_ASSERT_RETURN(index != 0, 0);
+            CARLA_SAFE_ASSERT_RETURN(index >= 'a', 0);
+            CARLA_SAFE_ASSERT_RETURN(index <= 'z', 0);
+            CARLA_SAFE_ASSERT_RETURN(value > 0, 0);
+            CARLA_SAFE_ASSERT_RETURN(ptr != nullptr, 0);
+            handlePreviewBufferData(static_cast<char>(index), static_cast<uint32_t>(value), (const void*)ptr);
+            break;
         }
 
         return ret;
 
         // unused for now
-        (void)index;
-        (void)value;
-        (void)ptr;
         (void)opt;
     }
 
@@ -903,13 +1155,18 @@ private:
     // carla as plugin does not implement lv2 parameter API yet, needed for feedback
     const bool kIgnoreParameters;
 
+    LV2_Atom_Forge  fAtomForge;
     uint32_t        fMidiEventCount;
     NativeMidiEvent fMidiEvents[kMaxMidiEvents];
 
     CarlaString fLastProjectPath;
     CarlaString fLoadedFile;
+    PreviewData fPreviewData;
+    volatile bool fNeedsNotifyFileChanged;
+    volatile int fPluginNeedsIdle;
 
     int fWorkerUISignal;
+    // -1 needs close, 0 idle, 1 stuff is writing??
 
     // -------------------------------------------------------------------
 
@@ -1112,14 +1369,21 @@ static LV2_Worker_Status lv2_work_resp(LV2_Handle instance, uint32_t size, const
     return instancePtr->lv2_work_resp(size, body);
 }
 
+static const LV2_Inline_Display_Image_Surface* lv2_idisp_render(LV2_Handle instance, uint32_t w, uint32_t h)
+{
+    // carla_debug("lv2_idisp_render(%p, %u, %u)", instance, w, h);
+    return instancePtr->lv2_idisp_render(w, h);
+}
+
 static const void* lv2_extension_data(const char* uri)
 {
     carla_debug("lv2_extension_data(\"%s\")", uri);
 
-    static const LV2_Options_Interface  options  = { lv2_get_options, lv2_set_options };
-    static const LV2_Programs_Interface programs = { lv2_get_program, lv2_select_program };
-    static const LV2_State_Interface    state    = { lv2_save, lv2_restore };
-    static const LV2_Worker_Interface   worker   = { lv2_work, lv2_work_resp, nullptr };
+    static const LV2_Options_Interface  options     = { lv2_get_options, lv2_set_options };
+    static const LV2_Programs_Interface programs    = { lv2_get_program, lv2_select_program };
+    static const LV2_State_Interface    state       = { lv2_save, lv2_restore };
+    static const LV2_Worker_Interface   worker      = { lv2_work, lv2_work_resp, nullptr };
+    static const LV2_Inline_Display_Interface idisp = { lv2_idisp_render };
 
     if (std::strcmp(uri, LV2_OPTIONS__interface) == 0)
         return &options;
@@ -1129,6 +1393,8 @@ static const void* lv2_extension_data(const char* uri)
         return &state;
     if (std::strcmp(uri, LV2_WORKER__interface) == 0)
         return &worker;
+    if (std::strcmp(uri, LV2_INLINEDISPLAY__interface) == 0)
+        return &idisp;
 
     return nullptr;
 }
