@@ -1,6 +1,6 @@
 /*
  * Carla State utils
- * Copyright (C) 2012-2018 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2020 Filipe Coelho <falktx@falktx.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -192,6 +192,7 @@ CarlaStateSave::CarlaStateSave() noexcept
       binary(nullptr),
       uniqueId(0),
       options(0x0),
+      temporary(false),
 #ifndef BUILD_BRIDGE_ALTERNATIVE_ARCH
       active(false),
       dryWet(1.0f),
@@ -454,7 +455,8 @@ bool CarlaStateSave::fillFromXmlElement(const XmlElement* const xmlElement)
                         {
                             const int ctrl(pText.getIntValue());
                             if (ctrl > CONTROL_INDEX_NONE && ctrl <= CONTROL_INDEX_MAX_ALLOWED)
-                                stateParameter->mappedControlIndex = static_cast<int16_t>(ctrl);
+                                if (ctrl != CONTROL_INDEX_MIDI_LEARN)
+                                    stateParameter->mappedControlIndex = static_cast<int16_t>(ctrl);
                         }
                         else if (pTag == "MappedMinimum")
                         {
